assert(require('StrixStore'))
assert(require('lsqlite3'))

nbSamples = 1*1000
nbSelect = 1000

wordLength = 10
knowsMin = 5
knowsMax = 15

math.randomseed(2010)
	
print('------------------ create RDF store ------------')

os.remove('release.db')
rdf.open{
	file = 'release.db', 
	uri = 'http://imac/',
	poolSize = 100*1024,
	quantum = 	512*1024*1024,
	initIndex	= 8*1024*1024,
	gcThreshold = 0,
	safe= false,
	noBuffer= false,
	writeThrough = true,
	truncate = true
	}
	

print('------------------ create Sqlite database ------------')
		
os.remove('sqlite.db')
sql = sqlite3.open('sqlite.db')

sql:exec[[
	PRAGMA encoding = "UTF-8"; 
	PRAGMA synchronous =  NORMAL;
	PRAGMA temp_store = FILE;
	CREATE TABLE person (uri TEXT PRIMARY KEY, name TEXT, age INTEGER,UNIQUE(name));
	CREATE TABLE knows(personUri TEXT, knowsUri TEXT,UNIQUE(personUri,knowsUri));
	CREATE INDEX knows_i1 ON knows(personUri);
	CREATE INDEX knows_i2 ON knows(knowsUri);
]]


print('------------------ building datas ------------')

function RandomWord()
	local w = ''
	for i = 1,wordLength do 
		w = w .. string.char(math.random(65,90) )
	end
	return w
end

------ build random words
words = {}
for i=1,nbSamples do
	words[i] = RandomWord()
end

local sqlCmd = 'BEGIN TRANSACTION;'
local rdfCmd = '@prefix : <http://imac/>. @prefix  foaf: <http://xmlns.com/foaf/0.1/>. \n'
local rdfCmd2 = '@prefix : <http://imac/>. @prefix  foaf: <http://xmlns.com/foaf/0.1/>. \n'

for i=1,nbSamples do
	local w = words[i]
	local age = math.random(18,65)
	local nbRelation = math.random(knowsMin,knowsMax)

	sqlCmd = sqlCmd .. 'INSERT INTO person VALUES("#person'..i..'","'..w..'",'..age..');'
	rdfCmd = rdfCmd .. ':person'..i..' a foaf:Person; foaf:name "'..w..'"; foaf:age '..age..' . \n'

	for k=1,nbRelation do
		local idRelation = math.random(1,nbSamples)
		sqlCmd = sqlCmd .. 'INSERT OR IGNORE INTO knows VALUES("#person'..i..'","#person'..idRelation..'");'
		rdfCmd2 = rdfCmd2 .. ':person'..i..' foaf:knows :person'..idRelation..'. \n'
	end
end

sqlCmd = sqlCmd .. 'END TRANSACTION;'



print('------------------ start ------------')

local t0 = os.clock()
sql:exec(sqlCmd)
print('SQLITE INSERT =',os.clock()-t0)

local t0 = os.clock()
rdf.graph.add('',rdfCmd)
rdf.graph.create('http://imac/knows/',rdfCmd2)
print('RDF turtle INSERT =',os.clock()-t0)


--print('------------------ dump ------------')
--for row in sql:nrows("SELECT * FROM person") do
  --print(row.id, row.name, row.age)
--end
--
--rdf.graph.export{uri=''}
--
-------- SELECT TEST 

local aName = words[ math.random(1,nbSamples) ]

local sqlQuery = [[
	SELECT DISTINCT p.name AS name FROM person,knows,person AS p
	WHERE person.name="]]..aName..[[" AND knows.personUri=person.uri 
	AND p.uri=knows.knowsUri
	]]


local rdfQuery = [[
	PREFIX  foaf: <http://xmlns.com/foaf/0.1/>
	SELECT DISTINCT ?name
	FROM NAMED <http://imac/knows/>
	WHERE {
		?x foaf:name "]]..aName..[[";
		GRAPH <http://imac/knows/>  {?x foaf:knows ?id }.
		?id foaf:name ?name
		}
]]


local t0 = os.clock()

for i=1,nbSelect do
	for row in sql:nrows(sqlQuery) do
	  --print(row.name)
	end
end

print('SQL SELECT  =',os.clock()-t0)

local t0 = os.clock()

for i=1,nbSelect do
	for name in rdf.sparql(rdfQuery) do
		--print(name)
	end
end

print('RDF SPARQL =',os.clock()-t0)

print('------------ SECOND SELECT TEST ----')

local sqlQuery = [[
	SELECT DISTINCT knows.personUri AS x,k.personUri AS y FROM knows,knows AS k 
	WHERE knows.knowsUri=k.knowsUri AND knows.personUri<>k.personUri
	]]

local rdfQuery = [[
	PREFIX  foaf: <http://xmlns.com/foaf/0.1/>
	SELECT ?x ?y
	FROM <http://imac/knows/>
	WHERE {
		?x foaf:knows ?z.
		?y foaf:knows ?z.
		FILTER(?x != ?y)
		}
]]

local t0 = os.clock()

for i=1,1 do
	for row in sql:nrows(sqlQuery) do
	  --print(row.x,row.y)
	end
end

print('SQL SELECT  =',os.clock()-t0)


local t0 = os.clock()

for i=1,1 do
	for name in rdf.sparql(rdfQuery) do
		--print(name)
	end
end

print('RDF SPARQL =',os.clock()-t0)
